#include "MySequences.h"

MySequences::MySequences(IMSSystem& ims) : myiMS(ims) {}

ImageSequence* MySequences::CreateSequence()
{
	seq_list.push_back(ImageSequence());
	return &seq_list.back();
}

bool MySequences::DownloadSequence(ImageSequence& seq)
{
	seq_dl.push_back(std::make_shared<SequenceDownload>(myiMS, seq));
	return seq_dl.back()->Download();
}

ImageSequence& MySequences::FindSequence(std::array<uint8_t, 16>& uuid)
{
	auto it = std::find_if(std::begin(seq_list), std::end(seq_list), [&](const ImageSequence& seq) { return seq.GetUUID() == uuid; });
	if (it != std::end(seq_list))
		return *it;
	return seq_list[0];
}

int MySequences::Count() const {
	return seq_list.size();
}

ImageSequence& MySequences::FindSequence(int index) {
	if (index > this->Count())
		return seq_list[0];
	else
		return seq_list[index];
}

std::string MySequences::SequenceTermString(iMS::SequenceTermAction action)
{
	switch (action) {
	case SequenceTermAction::DISCARD: return "DISCARD";
	case SequenceTermAction::RECYCLE: return "RECYCLE";
	case SequenceTermAction::STOP_DISCARD: return "STOP & DISCARD";
	case SequenceTermAction::STOP_RECYCLE: return "STOP & RECYCLE";
	case SequenceTermAction::REPEAT: return "REPEAT";
	case SequenceTermAction::REPEAT_FROM: return "REPEAT FROM ENTRY";
	case SequenceTermAction::INSERT: return "INSERT";
	case SequenceTermAction::STOP_INSERT: return "STOP & INSERT";
	default: return "UNKNOWN";
	}
}
