#include "ConnectionList.h"
#include "IMSSystem.h"
#include "Image.h"
#include "ImageOps.h"
#include "ToneBuffer.h"

#include <iostream>
#include <conio.h>
#include <chrono>

#ifndef IMS_ENHANCED_SEQUENCES__
#define IMS_ENHANCED_SEQUENCES__

// All API classes are defined in the iMS namespace.  For convenience, we can declare this here
using namespace iMS;

/// X Axis Centre Frequency
const MHz freq_centre_X = 100.0;
/// Y Axis Centre Frequency
const MHz freq_centre_Y = 100.0;
/// X Axis frequency increment per Image 'pixel'
const MHz freq_interval_X = 3.0;
/// Y Axis frequency increment per Image 'pixel'
const MHz freq_interval_Y = 3.0;

/// Number of images to generate from font map & download to iMS
const int number_of_images_in_group = 50;
/// Default processing clock rate when using internal clock
const kHz image_clock_rate = 1.0;
/// Bypass any font characters that result in an Image with fewer than this number of ImagePoints
const int min_image_size = 10;
/// Number of tones to synthesise into Tone Buffer
const int n_tones = 10;

/// Class definition for global access - use this For Single instance communications objects with the iMS
class MyIMSAccess {
public:
	// Global access to single instance SignalPath class
	SignalPath& sigpath() { return this->_sigpath; }
	// Global access to single instance SequenceManager class
	SequenceManager& seqMgr() { return this->_seqMgr; }
	MyIMSAccess(std::shared_ptr<IMSSystem> myiMS) : _sigpath(*myiMS), _seqMgr(*myiMS), thisIMS(myiMS) {}
	~MyIMSAccess() {}
private:
	MyIMSAccess(const MyIMSAccess&);  // Non-copyable
	const MyIMSAccess& operator =(const MyIMSAccess&);  // Non-assignable
	SignalPath _sigpath;
	SequenceManager _seqMgr;
	// Keep a reference to the IMS System to avoid it being deleted before us
	std::shared_ptr<IMSSystem> thisIMS;
};

extern std::unique_ptr < MyIMSAccess > glbl;

// Function definitions for Font Map / Tone Map creation
void createFontMapImageGroup();
void downloadFontMapToIMS(std::shared_ptr<IMSSystem>);
void createAndDownloadToneMap(std::shared_ptr<IMSSystem>);
iMS::Image& getImage(int index);
const iMS::Image& getImage(std::array<uint8_t, 16> uuid);
bool matchesToneBuffer(std::array<uint8_t, 16> uuid);
const ToneBuffer& getToneMap();

// Forward declarations
class MySequences;
class SequenceSupervisor;

// Access to MySequence object
extern std::unique_ptr < MySequences > myseq;

// Functions for controlling the curses menu system
void DrawQueue();
void DrawStatus(SequenceSupervisor& sup);
void UpdateStatus(SequenceSupervisor& sup);
int RunMenu();
void SequenceMenuInit();
void SequenceMenuDeinit();

#endif